"""
Demo adapter for IDocument.

This adapter has no serious functionality other than demonstrating how to
implement a custom indexing data provider. The implementation of this adapter
just returns the indexable data for IDocument 1:1 without further
modifications.

See api.COLLECTION_SCHEMA for the list of available fields and implementation
of api.indexable_content() for details.
"""


class DocumentIndexer:
    """Typesense indexer for IDocument"""

    def __init__(self, context):
        self.context = context

    def get_indexable_content(self, indexable_content):
        """Return indexable content for IDocument"""

        # you can modify the dict with the data to be indexed according to your
        # own needs.

        # indexable_content["text"] = "foo bar"
        return indexable_content



"""
The default Typesense collection schema
"""

import json


COLLECTION_SCHEMA = {
    "name": None,
    "fields": [
        {"name": "path", "type": "string"},
        {"name": "id", "type": "string"},
        {"name": "title", "type": "string"},
        {"name": "description", "type": "string"},
        {"name": "headlines", "type": "string"},
        {"name": "text", "type": "string"},
        {"name": "language", "type": "string", "facet": True},
        {"name": "portal_type", "type": "string", "facet": True},
        {"name": "review_state", "type": "string", "facet": True},
        {"name": "subject", "type": "string[]", "facet": True},
        {"name": "created", "type": "string", "facet": False},
        {"name": "modified", "type": "string", "facet": False},
        {"name": "effective", "type": "string", "facet": False},
        {"name": "expires", "type": "string", "facet": False},
        {"name": "document_type_order", "type": "int32"},
        {"name": "_indexed", "type": "string"},
        {"name": "all_paths", "type": "string[]", "facet": False},
    ],
    "default_sorting_field": "document_type_order",
    "token_separators": ["-"],
    "attributesToSnippet": [
        "title",
        "description",
        "text:20",
    ],
    "attributesToHighlight": [
        "title",
        "description",
        "text:20",
    ],
}


COLLECTION_SCHEMA_JSON = json.dumps(COLLECTION_SCHEMA, indent=2)

# CRLF separated list of review_states to be indexes
DEFAULT_REVIEW_STATES_TO_INDEX = "published"



from . import LOG
from huey import SqliteHuey

import typesense


huey = SqliteHuey(filename="/tmp/demo.db")


@huey.task()
def ts_index(ts_client, collection, data, document_id, document_path):

    LOG.debug(f"Indexing {document_id} : {document_path}")

    try:
        ts_client.collections[collection].documents.upsert(data)
    except typesense.exceptions.ObjectNotFound:
        LOG.error(f"Collection {collection} does not seem to exist")
        raise


@huey.task()
def ts_unindex(ts_client, collection, document_id, document_path):

    LOG.debug(f"Unindexing {document_id} : {document_path}")

    try:
        ts_client.collections[collection].documents[document_id].delete()
    except typesense.exceptions.ObjectNotFound:
        pass



"""
Indexing adapter via Apache Tika for File content
"""


from plone import api
from zopyx.typesense.interfaces import ITypesenseSettings

import os
import tempfile
import tika
import tika.parser

from .. import LOG

tika.initVM()

class FileIndexer:
    """Typesense indexer for IDocument"""

    def __init__(self, context):
        self.context = context

    def get_indexable_content(self, indexable_content):
        """Return indexable content for IFile"""

        # index content with Tika, if available
        tika_url = api.portal.get_registry_record("tika_url", ITypesenseSettings)
        if not tika_url:
            return indexable_content

        # save indexable content to temporary file
        tmp_fn = tempfile.mktemp()
        with open(tmp_fn, "wb") as fp:
            fp.write(self.context.file.data)

        # send temporary file to Apache Tika
        try:
            parsed = tika.parser.from_file(tmp_fn, serverEndpoint=tika_url)
        except Exception as e:
            LOG.exception("Unable to interact with Tika", exc_info=True)
            raise 
        finally:
            os.unlink(tmp_fn)

        indexable_content["text"] += parsed["content"]
        return indexable_content



import furl
import os
import pprint
import typer
import typesense


app = typer.Typer()


DEFAULT_SCHEMA = {"name": "companies", "fields": [{"name": ".*", "type": "auto"}]}

DEFAULT_URL = os.environ.get("TS_URL")
DEFAULT_API_KEY = os.environ.get("TS_API_KEY")


@app.command()
def list(url=typer.Option(DEFAULT_URL), api_key=typer.Option(DEFAULT_API_KEY)):

    f = furl.furl(url)
    nodes = [dict(host=f.host, port=f.port, protocol=f.scheme)]
    client = typesense.Client(dict(api_key=api_key, nodes=nodes))
    result = client.collections.retrieve()
    for row in result:
        print(f"{row['name']:20s} {row['num_documents']}")


@app.command()
def create(
    collection: str,
    url=typer.Option(DEFAULT_URL),
    api_key=typer.Option(DEFAULT_API_KEY),
):

    f = furl.furl(url)
    nodes = [dict(host=f.host, port=f.port, protocol=f.scheme)]
    client = typesense.Client(dict(api_key=api_key, nodes=nodes))

    schema = DEFAULT_SCHEMA.copy()
    schema["name"] = collection
    client.collections.create(schema)


@app.command()
def drop(
    collection: str,
    url=typer.Option(DEFAULT_URL),
    api_key=typer.Option(DEFAULT_API_KEY),
):

    f = furl.furl(url)
    nodes = [dict(host=f.host, port=f.port, protocol=f.scheme)]
    client = typesense.Client(dict(api_key=api_key, nodes=nodes))

    client.collections[collection].delete()


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
"""Init and utils."""
from huey.bin.huey_consumer import load_huey
from huey.consumer_options import ConsumerConfig
from zope.i18nmessageid import MessageFactory
from zopyx.plone.persistentlogger.file_logger import get_logger

import threading


_ = MessageFactory("zopyx.typesense")

LOG = get_logger("zopyx.typesense")

# Start huey consumer as thread

from huey.consumer import Consumer

import signal


# monkey-patch huey
def my_set_signal_handlers(self):
    """Ignore signal errors from Huey"""
    try:
        signal.signal(signal.SIGTERM, self._handle_stop_signal)
        signal.signal(signal.SIGINT, signal.default_int_handler)
        if hasattr(signal, "SIGHUP"):
            signal.signal(signal.SIGHUP, self._handle_restart_signal)
    except ValueError:
        LOG.warning("Huey signal exception ignored")


Consumer._set_signal_handlers = my_set_signal_handlers


consumer_options = {
    "backoff": 1.15,
    "check_worker_health": True,
    "extra_locks": None,
    "flush_locks": False,
    "health_check_interval": 10,
    "initial_delay": 0.1,
    "max_delay": 10.0,
    "periodic": True,
    "scheduler_interval": 1,
    "worker_type": "thread",
    "workers": 4,
    "logfile": "huey.log",
    "verbose": False,
}

h = load_huey("zopyx.typesense.huey_tasks.huey")

config = ConsumerConfig(**consumer_options)
config.validate()
config.setup_logger()
consumer = h.create_consumer(**config.values)

th = threading.Thread(target=consumer.run)
th.start()

LOG.info("Started Huey consumer thread")



from .api import API
from plone import api
from zopyx.typesense import LOG
from zopyx.typesense.interfaces import ITypesenseSettings

import time


def can_index():
    """Check if the Typesense registry settings are available"""

    try:
        api.portal.get_registry_record("collection", ITypesenseSettings)
    except:
        return False

    enabled = api.portal.get_registry_record("enabled", ITypesenseSettings)
    if not enabled:
        site_id = api.portal.get().getId()
        LOG.warning(f"Typesense indexing is disabled for Plone site {site_id}")
    return enabled


def remove_content(context, event):
    """Async removal of content"""

    if not can_index():
        return

    ts = time.time()
    ts_api = API()
    ts_api.unindex_document(context)
    duration = (time.time() - ts) * 1000
    LOG.debug(
        f"Unindexing {context.getId(), context.absolute_url(1)}, {duration:.3f} ms"
    )


def update_content(context, event):
    """Async indexing of content"""

    if not can_index():
        return

    ts = time.time()
    ts_api = API()

    ts_api.index_document(context)
    duration = (time.time() - ts) * 1000
    LOG.debug(f"Indexing {context.getId(), context.absolute_url(1)}, {duration:.3f} ms")


def workflow_transition(context, event):
    """Index/unindex content upon workflow transition"""

    review_state = api.content.get_state(context)
    try:
        review_states_to_index = api.portal.get_registry_record(
            "review_states_to_index", ITypesenseSettings
        )
    except KeyError:
        return
    if not review_state or review_state in review_states_to_index:
        update_content(context, None)
    else:
        remove_content(context, None)



# -*- coding: utf-8 -*-


from plone.app.registry.browser import controlpanel
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zopyx.typesense import _
from zopyx.typesense.interfaces import ITypesenseSettings

import json


class TypesenseSettingsEditForm(controlpanel.RegistryEditForm):

    schema = ITypesenseSettings
    label = _(u"Typesense settings")
    description = _(u"")

    def updateFields(self):
        super(TypesenseSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(TypesenseSettingsEditForm, self).updateWidgets()


class TypesenseSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = TypesenseSettingsEditForm

    @property
    def settings(self):
        """Returns setting as dict"""
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ITypesenseSettings)
        result = dict()
        for name in settings.__schema__:
            result[name] = getattr(settings, name)
        return result

    def settings_json(self):
        """Returns setting as JSON"""
        return json.dumps(self.settings)



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    applyProfile,
    FunctionalTesting,
    IntegrationTesting,
    PLONE_FIXTURE,
    PloneSandboxLayer,
)
from plone.testing import z2

import zopyx.typesense


class ZopyxTypesenseLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.dexterity

        self.loadZCML(package=plone.app.dexterity)
        import plone.restapi

        self.loadZCML(package=plone.restapi)
        self.loadZCML(package=zopyx.typesense)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "zopyx.typesense:default")


ZOPYX_TYPESENSE_FIXTURE = ZopyxTypesenseLayer()


ZOPYX_TYPESENSE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(ZOPYX_TYPESENSE_FIXTURE,),
    name="ZopyxTypesenseLayer:IntegrationTesting",
)


ZOPYX_TYPESENSE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(ZOPYX_TYPESENSE_FIXTURE,),
    name="ZopyxTypesenseLayer:FunctionalTesting",
)


ZOPYX_TYPESENSE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        ZOPYX_TYPESENSE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name="ZopyxTypesenseLayer:AcceptanceTesting",
)



from .huey_tasks import ts_index, ts_unindex
from datetime import datetime
from plone import api
from plone.app.contenttypes.indexers import SearchableText
from plone.app.textfield import RichText
from plone.dexterity.utils import iterSchemata
from zope.component import ComponentLookupError, getAdapter
from zope.interface.interfaces import ComponentLookupError
from zope.schema import getFields
from zopyx.typesense import _, LOG
from zopyx.typesense.interfaces import ITypesenseIndexDataProvider, ITypesenseSettings

import furl
import html_text
import json
import lxml.html
import typesense
import zope.schema


def html2text(html):

    if isinstance(html, bytes):
        html = html.decode("utf8")
    tree = html_text.parse_html(html)
    return html_text.extract_text(tree)


def headlines_from_html(html):

    if isinstance(html, bytes):
        html = html.decode("utf8")

    root = lxml.html.fromstring(html)

    result = []

    for node in root.xpath(
        "//*[self::h2 or self::h3 or self::h4 or self::h5 or self::h6 or self::h7]"
    ):
        if node.text:
            result.append(node.text)

    result = " ".join(result)
    return result


class API:
    @property
    def collection(self):
        """Return collection name from registry"""
        collection = api.portal.get_registry_record("collection", ITypesenseSettings)
        return collection

    def index_document(self, obj, collection=None):
        """Index document `obj`"""

        data = self.indexable_content(obj)
        if not data:
            return

        target_collection = collection if collection else self.collection

        ts_index(
            ts_client=self.get_typesense_client(),
            collection=target_collection,
            data=data,
            document_id=self.document_id(obj),
            document_path=self.document_path(obj),
        )

    def unindex_document(self, obj):
        """Unindex document `obj`"""

        ts_unindex(
            ts_client=self.get_typesense_client(),
            collection=self.collection,
            document_id=self.document_id(obj),
            document_path=self.document_path(obj),
        )

    def indexable_content(self, obj):
        """Return dict with indexable content for `obj`"""

        # review states
        review_states_to_index = api.portal.get_registry_record(
            "review_states_to_index", ITypesenseSettings
        )
        review_states_to_index = [
            s.strip() for s in review_states_to_index.split("\n") if s.strip()
        ]

        ignore_review_state = False
        try:
            review_state = api.content.get_state(obj)
        except:
            review_state = ""
            ignore_review_state = True

        if not ignore_review_state and not review_state in review_states_to_index:
            # don't index content without proper review state
            return

        # language
        default_language = api.portal.get_default_language()
        language = obj.Language() or default_language

        document_id = self.document_id(obj)

        d = dict()
        d["id"] = document_id
        d["id_original"] = obj.getId()
        d["title"] = obj.Title()
        d["description"] = obj.Description()
        d["language"] = language
        d["portal_type"] = obj.portal_type
        d["review_state"] = review_state
        d["path"] = self.document_path(obj)
        d["all_paths"] = self.document_path_items(obj)
        d["created"] = obj.created().ISO8601()
        d["modified"] = obj.modified().ISO8601()
        d["effective"] = obj.effective().ISO8601()
        d["expires"] = obj.expires().ISO8601()
        d["subject"] = obj.Subject()
        d["uid"] = obj.UID()
        d["document_type_order"] = 0
        d["_indexed"] = datetime.utcnow().isoformat()

        use_searchabletext = api.portal.get_registry_record(
            "use_searchabletext", ITypesenseSettings
        )
        if use_searchabletext:
            # use Plone's SearchableText implemenation
            indexable_text = SearchableText(obj)
        else:
            # or our own indexable text content
            indexable_text = []
            indexable_headlines = []

            fields = {}
            schemes = iterSchemata(obj)
            for schema in schemes:
                fields.update(getFields(schema))

            for name, field in fields.items():
                if isinstance(field, RichText):
                    text = getattr(obj, name)
                    if isinstance(text, str):
                        indexable_text.append(text)
                    else:
                        if text and text.output:
                            indexable_text.append(html2text(text.output))
                            indexable_headlines.append(headlines_from_html(text.output))
                elif isinstance(field, (zope.schema.Text, zope.schema.TextLine)):
                    text = getattr(obj, name)
                    indexable_text.append(text)

            indexable_text = [text for text in indexable_text if text]
            indexable_text = " ".join(indexable_text)
            indexable_headlines = [text for text in indexable_headlines if text]
            indexable_headlines = " ".join(indexable_headlines)

        d["text"] = indexable_text
        d["headlines"] = indexable_headlines

        # Check if there is an adapter for the given content type interface
        # implementing ITypesenseIndexDataProvider for modifying the index data
        # dict.

        try:
            adapter = getAdapter(obj, ITypesenseIndexDataProvider)
        except ComponentLookupError:
            adapter = None

        if adapter:
            d = adapter.get_indexable_content(d)

        return d

    def indexed_content(self, obj):
        """Return the indexed content in Typesense for the given `obj`"""

        client = self.get_typesense_client()
        document_id = self.document_id(obj)

        try:
            document = (
                client.collections[self.collection].documents[document_id].retrieve()
            )
        except typesense.exceptions.ObjectNotFound:
            document = {}

        return document

    def document_id(self, obj):
        """Return the content id for the given `obj`"""

        site_id = api.portal.get().getId()
        obj_id = f"{site_id}-{obj.UID()}"
        return obj_id

    def document_path(self, obj):
        """Return the content path for the given `obj`"""

        site_path = "/".join(api.portal.get().getPhysicalPath())
        obj_path = "/".join(obj.getPhysicalPath())
        rel_path = obj_path.replace(site_path, "")
        rel_path = rel_path.lstrip("/")
        return rel_path

    def document_path_items(self, obj):
        """Return all possible prefix path components of the given object"""

        document_path = self.document_path(obj)
        document_path_items = document_path.split("/")
        all_paths = list()
        for i in range(len(document_path_items) + 1):
            items = "/".join(document_path_items[:i])
            if not items.startswith("/"):
                items = "/" + items
            if not items in all_paths:
                all_paths.append(items)
        return all_paths

    def exists_collection(self, collection):
        """Check if collection exists"""

        client = self.get_typesense_client()
        all_collections = [
            collection["name"] for collection in client.collections.retrieve()
        ]
        return collection in all_collections

    def create_collection(self, temporary=False):
        """Create collection"""

        client = self.get_typesense_client()
        collection = self.collection
        if temporary:
            collection = collection + "-" + datetime.utcnow().isoformat()

        if self.exists_collection(collection):
            raise RuntimeError(f"Collection `{collection}` already exists")

        collection_schema = api.portal.get_registry_record(
            "collection_schema", ITypesenseSettings
        )
        collection_schema = json.loads(collection_schema)
        collection_schema["name"] = collection

        client.collections.create(collection_schema)
        LOG.info(f"Created Typesense collection {collection}")
        return collection

    def drop_collection(self, collection_name=None):
        """Drop collection"""

        collection = collection_name if collection_name else self.collection

        if self.exists_collection(collection):

            client = self.get_typesense_client()
            try:
                client.collections[collection].delete()
                LOG.info(f"Deleted Typesense collection {collection}")
            except Exception as e:
                LOG.exception(f"Could not delete Typesense collection {collection}")
                raise

    def alias_collection(self, collection, alias_collection):
        """alias `alias_collection` to `collection`"""

        client = self.get_typesense_client()
        client.aliases.upsert(collection, dict(collection_name=alias_collection))
        LOG.info(f"Creating alias '{collection}' for {alias_collection}")

    def remove_obsolete_collections(self, collection_prefix):
        """Remove obsolete collections with prefix `collection_prefix`"""

        client = self.get_typesense_client()
        all_collections = client.collections.retrieve()
        names = [
            d["name"]
            for d in all_collections
            if d["name"].startswith(collection_prefix + "-")
        ]
        names = sorted(names)
        names_to_remove = names[:-1]
        for name in names_to_remove:
            self.drop_collection(name)

    def collection_stats(self):
        """Get collection statistics"""

        client = self.get_typesense_client()
        try:
            result = client.collections[self.collection].retrieve()
        except Exception as e:
            LOG.error("Unable to fetch Typesense stats", exc_info=True)
            return {}

        result["created_at_str"] = datetime.fromtimestamp(
            result["created_at"]
        ).isoformat()
        return result

    def get_typesense_client(
        self,
    ):
        """Typesense client with full admin access"""
        return self.get_client(self.api_key)

    def get_typesense_search_client(self):
        """Typesense client with search access"""
        return self.get_client(self.search_api_key)

    @property
    def search_api_key(self):
        """Return search API key"""
        return api.portal.get_registry_record("search_api_key", ITypesenseSettings)

    @property
    def api_key(self):
        """Return admin API key"""
        return api.portal.get_registry_record("api_key", ITypesenseSettings)

    def get_client(self, api_key):
        """Get Typesense client for given API key"""

        if not api_key:
            raise ValueError(_("No Typesense API key(s) configured"))

        client = typesense.Client(
            {
                "api_key": api_key,
                "nodes": self.nodes,
                "connection_timeout_seconds": 10,
            }
        )
        return client

    @property
    def nodes(self):
        """Return a list of Typesense nodes (used by the search UI)"""

        try:
            node1_url = api.portal.get_registry_record("node1_url", ITypesenseSettings)
            node2_url = api.portal.get_registry_record("node2_url", ITypesenseSettings)
            node3_url = api.portal.get_registry_record("node3_url", ITypesenseSettings)
        except (KeyError, ComponentLookupError):
            return None

        nodes = list()
        for url in (node1_url, node2_url, node3_url):
            if not url:
                continue

            f = furl.furl(url)
            nodes.append(dict(host=f.host, port=f.port, protocol=f.scheme))

        return nodes

    def export_documents(self, format="jsonl"):
        """Export all documents of collection as JSONlines"""
        client = self.get_typesense_client()
        result = client.collections[self.collection].documents.export()
        if format == "jsonl":
            return result
        else:
            # JSON
            result = [json.loads(jline) for jline in result.splitlines()]
            return json.dumps(result, indent=2)

    def search(self, query, per_page=25, page=1):

        client = self.get_typesense_client()
        search_params = {
            "q": query,
            "query_by": "text",
            "per_page": per_page,
            "page": page,
            #            'sort_by': 'id2:desc',
            #            'facet_by': ['language,area,document_type,societies,specifications_str,specifications2_str,topic_str']
        }

        LOG.info(search_params)
        result = client.collections[self.collection].documents.search(search_params)
        result["pages"] = (
            int(result["found"] / result["request_params"]["per_page"]) + 1
        )
        return result

    def snapshot(self):
        """Snapshot typesense database.
        Cavecat: If Typesense is running with a Docker container,
        the snapshot will be created inside the container unless you configure
        a volume mapping.
        """

        client = self.get_typesense_client()
        snapshot_path = f"{self.collection}-{datetime.utcnow().isoformat()}.snapshot"
        client.operations.perform("snapshot", {"snapshot_path": snapshot_path})
        return snapshot_path

    def cluster_data(self):
        """Return metrics, stats etc. from Typesense"""

        client = self.get_typesense_client()

        try:
            # cluster
            metrics = client.metrics.retrieve()
            stats = client.stats.retrieve()
            health = client.health.retrieve()
            return dict(metrics=metrics, health=health, stats=stats)
        except AttributeError:
            # standalone
            return None



# -*- coding: utf-8 -*-
"""Module where all interfaces, events and exceptions live."""

from .config import COLLECTION_SCHEMA_JSON, DEFAULT_REVIEW_STATES_TO_INDEX
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zopyx.typesense import _


class IBrowserLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class ITypesenseSettings(Interface):
    """Connector settings"""

    enabled = schema.Bool(
        title=_("Typesense integration enabled"),
        default=True,
    )

    collection = schema.TextLine(
        title=_("Name of Typesense collection"),
        default="typesense",
        required=True,
    )

    api_key = schema.TextLine(
        title=_("Typesense Admin API key"),
        default="",
        required=True,
    )

    search_api_key = schema.TextLine(
        title=_("Typesense search API key"),
        default="",
        required=True,
    )

    node1_url = schema.TextLine(
        title=_("URL of Typesense node 1"),
        description=_("URL node 1"),
        default="http://localhost:8108",
        required=True,
    )

    node2_url = schema.TextLine(
        title=_("URL of Typesense node 2"),
        description=_("URL node 2"),
        required=False,
    )

    node3_url = schema.TextLine(
        title=_("URL of Typesense node 3"),
        description=_("URL node 3"),
        required=False,
    )

    tika_url = schema.TextLine(
        title=_("URL of Tika server for indexing office formats"),
        description=_("URL Tika server"),
        required=False,
    )

    review_states_to_index = schema.Text(
        title=_("Review states to index "),
        default=DEFAULT_REVIEW_STATES_TO_INDEX,
        required=True,
    )

    collection_schema = schema.Text(
        title=_("Collection schema"),
        default=COLLECTION_SCHEMA_JSON,
        required=True,
    )

    use_searchabletext = schema.Bool(
        title=_("Use SearchableText for indexing as default"),
        default=False,
    )


class ITypesenseIndexDataProvider(Interface):
    """Adapter for custom indexing"""

    def get_indexable_content(indexable_content):
        """This method get the default data dict with
        indexed_content (see api.py). The custom
        indexer method can modify or provide additional
        data to be indexed.

        Returns an updated dict of indexed_content
        """



from Products.Five.browser import BrowserView
from zopyx.typesense import _, LOG
from zopyx.typesense.api import API

import gzip
import json
import os
import plone.api
import progressbar
import time


class View(BrowserView):
    def recreate_collection(self):

        ts_api = API()

        ts_api.drop_collection()
        ts_api.create_collection()

        LOG.info(f"Created Typesense collection {ts_api.collection}")

        portal = plone.api.portal.get()
        plone.api.portal.show_message(
            _("Typesense collection dropped and recreated"),
            request=self.request,
        )
        self.request.response.redirect(portal.absolute_url() + "/@@typesense-admin")

    def indexed_content(self):
        """Return indexed content for current context object"""

        ts_api = API()
        document = ts_api.indexed_content(self.context)
        return document

    def export_documents(self, format="jsonl"):
        """Export all documents from current collection as JSONLines"""

        ts_api = API()
        result = ts_api.export_documents(format)

        self.request.response.setHeader("content-type", f"application/{format}")
        self.request.response.setHeader(
            "content-disposition",
            f"attachment; filename={ts_api.collection}.{format}",
        )
        return result

    def collection_information(self):
        """Retrieve Collection information"""

        ts_api = API()
        return ts_api.collection_stats()

    def reindex_all(self):
        """Reindex all"""

        ts = time.time()
        ts_api = API()
        #        ts_api.drop_collection()
        collection = ts_api.create_collection(temporary=True)

        catalog = plone.api.portal.get_tool("portal_catalog")
        brains = catalog()
        num_brains = len(list(brains))

        with progressbar.ProgressBar(max_value=num_brains) as pg:
            for i, brain in enumerate(brains):
                pg.update(i)
                obj = brain.getObject()
                ts_api.index_document(obj, collection)

        ts_api.drop_collection(ts_api.collection)
        ts_api.alias_collection(ts_api.collection, collection)
        ts_api.remove_obsolete_collections(ts_api.collection)

        duration = time.time() - ts
        LOG.info(
            f"All content submitted for reindexing ({num_brains} items), duration {duration:.2f} seconds"
        )

        portal = plone.api.portal.get()
        plone.api.portal.show_message(
            _(
                "All content submitted for reindexing. Results may/will show up delayed depending on the amount of documents!"
            ),
            request=self.request,
        )
        self.request.response.redirect(portal.absolute_url() + "/@@typesense-admin")

    def search_result(self):
        """Search UI for admin view"""
        ts_api = API()
        result = ts_api.search(
            query=self.request.form.get("query"), page=self.request.form.get("page", 1)
        )
        return result

    def import_demo_content(self):

        from plone.app.textfield.value import RichTextValue

        portal = plone.api.portal.get()

        LOG.info("Deleting folder: new")
        if "news" in portal.objectIds():
            plone.api.content.delete(portal.news)

        LOG.info("Creating folder: new")
        data_folder = plone.api.content.create(
            container=portal, type="Folder", id="news", title="news"
        )

        fn = os.path.dirname(__file__) + "/de-news.json"
        news = json.load(open(fn))

        with progressbar.ProgressBar(max_value=len(news)) as pg:
            for i, n in enumerate(news):
                pg.update(i)
                text = RichTextValue(n["text"], "text/html", "text/html")
                doc = plone.api.content.create(
                    type="News Item",
                    container=data_folder,
                    title=n["title"],
                    text=text,
                    language="de",
                )
                plone.api.content.transition(doc, "publish")

        plone.api.portal.show_message(
            _("Sample content imported into folder /news"),
            request=self.request,
        )
        self.request.response.redirect(portal.absolute_url() + "/@@typesense-admin")

    def snapshot(self):
        """Create a snapshot of the Typesense internal database"""

        ts_api = API()
        snapshot_name = ts_api.snapshot()

        portal = plone.api.portal.get()
        plone.api.portal.show_message(
            _(f"Snapshot taken ({snapshot_name})"),
            request=self.request,
        )
        self.request.response.redirect(portal.absolute_url() + "/@@typesense-admin")

    def cluster_data(self):
        """Return metrics, stats from Typesense"""

        ts_api = API()
        return ts_api.cluster_data()

    def current_path(self):
        """Return the current folder path relativ to the Plone site"""

        portal_path = plone.api.portal.get().absolute_url(1)
        context_path = self.context.absolute_url(1)
        context_path = context_path.replace(portal_path, "")
        if not context_path.startswith("/"):
            context_path = "/" + context_path
        return context_path

    def search_settings(self):
        """Typesense settings returned as JSON for dynamic search UI"""

        ts_api = API()

        settings = dict()
        settings["collection"] = ts_api.collection
        settings["api_key"] = ts_api.search_api_key
        settings["nodes"] = ts_api.nodes
        settings["query_by"] = "title,headlines,text"
        settings["query_weights"] = "4,2,1"

        self.request.response.setHeader("content-type", "application/json")
        return json.dumps(settings)

    def import_demo_content2(self):

        from plone.app.textfield.value import RichTextValue

        portal = plone.api.portal.get()

        if "demo" in portal.objectIds():
            plone.api.content.delete(portal.news)

        data_folder = plone.api.content.create(
            container=portal, type="Folder", id="demo", title="demo"
        )

        fn = os.path.dirname(__file__) + "/demo.json"
        news = json.load(open(fn))
        for n in news:
            text = RichTextValue(n["text"], "text/html", "text/html")
            doc = plone.api.content.create(
                type="News Item",
                container=data_folder,
                title=n["title"],
                text=text,
                language="de",
            )
            plone.api.content.transition(doc, "publish")

        plone.api.portal.show_message(
            _("Sample content imported into folder /demo"),
            request=self.request,
        )
        self.request.response.redirect(portal.absolute_url() + "/@@typesense-admin")
